@@ -1,7 +1,5 @@
 /*
  * Functions internal to the PV library.
- *
- * Copyright 2014 Andrew Wood, distributed under the Artistic License 2.0.
  */
 
 #ifndef _PV_INTERNAL_H
@@ -33,6 +31,7 @@ extern "C" {
 #define PV_DISPLAY_NAME		64
 #define PV_DISPLAY_BUFPERCENT	128
 #define PV_DISPLAY_OUTPUTBUF	256
+#define PV_DISPLAY_FINETA	512
 
 #define RATE_GRANULARITY	100000	 /* usec between -L rate chunks */
 #define REMOTE_INTERVAL		100000	 /* usec between checks for -R */
@@ -69,6 +68,7 @@ struct pvstate_s {
 	unsigned long long target_buffer_size;  /* buffer size (0=default) */
 	unsigned long long size;         /* total size of data */
 	double interval;                 /* interval between updates */
+	double delay_start;              /* delay before first display */
 	unsigned int watch_pid;		 /* process to watch fds of */
 	int watch_fd;			 /* fd to watch */
 	unsigned int width;              /* screen width */
@@ -91,8 +91,8 @@ struct pvstate_s {
 	struct timeval pv_sig_tstp_time; /* see pv_sig_tstp() / __cont() */
 	struct timeval pv_sig_toffset;		 /* total time spent stopped */
 	volatile sig_atomic_t pv_sig_newsize;	 /* whether we need to get term size again */
-	volatile sig_atomic_t pv_sig_abort;		 /* whether we need to abort right now */
-	volatile sig_atomic_t reparse_display;
+	volatile sig_atomic_t pv_sig_abort;	 /* whether we need to abort right now */
+	volatile sig_atomic_t reparse_display;	 /* whether to re-check format string */
 	struct sigaction pv_sig_old_sigpipe;
 	struct sigaction pv_sig_old_sigttou;
 	struct sigaction pv_sig_old_sigtstp;
@@ -113,23 +113,25 @@ struct pvstate_s {
 	char *display_buffer;
 	long display_buffer_size;
 	int lastoutput_length;		 /* number of last-output bytes to show */
-	unsigned char lastoutput_buffer[256];	 /* RATS: ignore (big enough) */
+	unsigned char lastoutput_buffer[256];
 	int prev_width;			 /* screen width last time we were called */
 	int prev_length;		 /* length of last string we output */
-	char str_name[512];		 /* RATS: ignore (big enough) */
-	char str_transferred[128];	 /* RATS: ignore (big enough) */
-	char str_bufpercent[128];	 /* RATS: ignore (big enough) */
-	char str_timer[128];		 /* RATS: ignore (big enough) */
-	char str_rate[128];		 /* RATS: ignore (big enough) */
-	char str_average_rate[128];	 /* RATS: ignore (big enough) */
-	char str_progress[1024];	 /* RATS: ignore (big enough) */
-	char str_lastoutput[512];	 /* RATS: ignore (big enough) */
-	char str_eta[128];		 /* RATS: ignore (big enough) */
+	char str_name[512];
+	char str_transferred[128];
+	char str_bufpercent[128];
+	char str_timer[128];
+	char str_rate[128];
+	char str_average_rate[128];
+	char str_progress[1024];
+	char str_lastoutput[512];
+	char str_eta[128];
+	char str_fineta[128];
 	unsigned long components_used;	 /* bitmask of components used */
 	struct {
 		const char *string;
 		int length;
 	} format[100];
+	unsigned char display_visible;	 /* set once anything written to terminal */
 
 	/********************
 	 * Cursor/IPC state *
@@ -145,7 +147,7 @@ struct pvstate_s {
 	int crs_noipc;			 /* set if we can't use IPC */
 #endif				/* HAVE_IPC */
 	int crs_lock_fd;		 /* fd of lockfile, -1 if none open */
-	char crs_lock_file[1024];	 /* RATS: ignore */
+	char crs_lock_file[1024];
 	int crs_y_start;		 /* our initial Y coordinate */
 
 	/*******************
@@ -218,6 +220,8 @@ struct pvwatchfd_s {
 };
 typedef struct pvwatchfd_s *pvwatchfd_t;
 
+void pv_error(pvstate_t, char *, ...);
+
 int pv_main_loop(pvstate_t);
 void pv_display(pvstate_t, long double, long long, long long);
 long pv_transfer(pvstate_t, int, int *, int *, unsigned long long, long *);
