@@ -1,13 +1,12 @@
 /*
  * Display functions.
- *
- * Copyright 2014 Andrew Wood, distributed under the Artistic License 2.0.
  */
 
 #include "pv-internal.h"
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdarg.h>
 #include <string.h>
 #include <ctype.h>
 #include <errno.h>
@@ -17,6 +16,24 @@
 #include <sys/ioctl.h>
 
 
+/*
+ * Output an error message.  If we've displayed anything to the terminal
+ * already, then put a newline before our error so we don't write over what
+ * we've written.
+ */
+void pv_error(pvstate_t state, char *format, ...)
+{
+	va_list ap;
+	if (state->display_visible)
+		fprintf(stderr, "\n");
+	fprintf(stderr, "%s: ", state->program_name);
+	va_start(ap, format);
+	vfprintf(stderr, format, ap);
+	va_end(ap);
+	fprintf(stderr, "\n");
+}
+
+
 /*
  * Fill in *width and *height with the current terminal size,
  * if possible.
@@ -158,8 +175,8 @@ static void pv__sizestr(char *buffer, int bufsize, char *format,
 			long double amount, char *suffix_basic,
 			char *suffix_bytes, int is_bytes)
 {
-	char sizestr_buffer[256];	 /* RATS: ignore (big enough) */
-	char si_prefix[8] = " ";	 /* RATS: ignore (big enough) */
+	char sizestr_buffer[256];
+	char si_prefix[8] = " ";
 	long double divider;
 	long double display_amount;
 	char *suffix;
@@ -189,9 +206,9 @@ static void pv__sizestr(char *buffer, int bufsize, char *format,
 		 * AIX blows up with %4.3Lg%.2s%.16s for some reason, so we
 		 * write display_amount separately first.
 		 */
-		char str_disp[64];	 /* RATS: ignore (OK) */
+		char str_disp[64];
 		sprintf(str_disp, "%4.3Lg", display_amount);
-		sprintf(sizestr_buffer, "%s%.2s%.16s" /* RATS: ignore */ ,
+		sprintf(sizestr_buffer, "%s%.2s%.16s",
 			str_disp, si_prefix, suffix);
 	}
 
@@ -284,6 +301,13 @@ static void pv__format_init(pvstate_t state)
 				state->format[segment].length = 0;
 				state->components_used |= PV_DISPLAY_ETA;
 				break;
+			case 'I':
+				state->format[segment].string =
+				    state->str_fineta;
+				state->format[segment].length = 0;
+				state->components_used |=
+				    PV_DISPLAY_FINETA;
+				break;
 			case 'A':
 				state->format[segment].string =
 				    state->str_lastoutput;
@@ -371,6 +395,15 @@ static void pv__format_init(pvstate_t state)
 	state->format[segment].length = 0;
 }
 
+/*
+ * Return the original value x so that it has been clamped between
+ * [min..max]
+ */
+static long bound_long(long x, long min, long max)
+{
+	return x < min ? min : x > max ? max : x;
+}
+
 
 /*
  * Return a pointer to a string (which must not be freed), containing status
@@ -490,14 +523,13 @@ static char *pv__format(pvstate_t state,
 	if (NULL == state->display_buffer) {
 		state->display_buffer_size = (2 * state->width) + 80;
 		if (state->name)
-			state->display_buffer_size += strlen(state->name);	/* RATS: ignore */
+			state->display_buffer_size += strlen(state->name);
 		state->display_buffer =
 		    malloc(state->display_buffer_size + 16);
 		if (NULL == state->display_buffer) {
-			fprintf(stderr, "%s: %s: %s\n",
-				state->program_name,
-				_("buffer allocation failed"),
-				strerror(errno));
+			pv_error(state, "%s: %s",
+				 _("buffer allocation failed"),
+				 strerror(errno));
 			state->exit_status |= 64;
 			return NULL;
 		}
@@ -513,7 +545,7 @@ static char *pv__format(pvstate_t state,
 	 * of the percentage. (Or lines, if --lines was given with --bytes).
 	 */
 	if (state->numeric) {
-		char numericprefix[128]; /* RATS: ignore (OK) */
+		char numericprefix[128];
 
 		numericprefix[0] = 0;
 
@@ -550,6 +582,7 @@ static char *pv__format(pvstate_t state,
 	state->str_progress[0] = 0;
 	state->str_lastoutput[0] = 0;
 	state->str_eta[0] = 0;
+	state->str_fineta[0] = 0;
 
 	/* If we're showing bytes transferred, set up the display string. */
 	if ((state->components_used & PV_DISPLAY_BYTES) != 0) {
@@ -622,18 +655,26 @@ static char *pv__format(pvstate_t state,
 				 state->size - state->initial_offset,
 				 elapsed_sec);
 
-		if (eta < 0)
-			eta = 0;
-
 		/*
 		 * Bounds check, so we don't overrun the suffix buffer. This
 		 * means the ETA will always be less than 100,000 hours.
 		 */
-		if (eta > (long) 360000000L)
-			eta = (long) 360000000L;
+		eta = bound_long(eta, 0, (long) 360000000L);
 
-		sprintf(state->str_eta, "%.16s %ld:%02ld:%02ld", _("ETA"),
-			eta / 3600, (eta / 60) % 60, eta % 60);
+		/*
+		 * If the ETA is more than a day, include a day count as
+		 * well as hours, minutes, and seconds.
+		 */
+		if (eta > 86400L) {
+			sprintf(state->str_eta,
+				"%.16s %ld:%02ld:%02ld:%02ld", _("ETA"),
+				eta / 86400, (eta / 3600) % 24,
+				(eta / 60) % 60, eta % 60);
+		} else {
+			sprintf(state->str_eta, "%.16s %ld:%02ld:%02ld",
+				_("ETA"), eta / 3600, (eta / 60) % 60,
+				eta % 60);
+		}
 
 		/*
 		 * If this is the final update, show a blank space where the
@@ -648,6 +689,68 @@ static char *pv__format(pvstate_t state,
 		}
 	}
 
+	/* ETA as clock time (as above) - set up the display string. */
+	if (((state->components_used & PV_DISPLAY_FINETA) != 0)
+	    && (state->size > 0)) {
+		/*
+		 * The ETA may be hidden by a failed ETA string
+		 * generation.
+		 */
+		int show_eta = 1;
+		time_t now = time(NULL);
+		time_t then;
+		struct tm *time_ptr;
+		char *time_format = NULL;
+
+		eta =
+		    pv__calc_eta(total_bytes - state->initial_offset,
+				 state->size - state->initial_offset,
+				 elapsed_sec);
+
+		/*
+		 * Bounds check, so we don't overrun the suffix buffer. This
+		 * means the ETA will always be less than 100,000 hours.
+		 */
+		eta = bound_long(eta, 0, (long) 360000000L);
+
+		/*
+		 * Only include the date if the ETA is more than 6 hours
+		 * away.
+		 */
+		if (eta > (long) (6 * 3600)) {
+			time_format = "%Y-%m-%d %H:%M:%S";
+		} else {
+			time_format = "%H:%M:%S";
+		}
+
+		then = now + eta;
+		time_ptr = localtime(&then);
+
+		if (NULL == time_ptr) {
+			show_eta = 0;
+		} else {
+			/* Localtime keeps data stored in a
+			 * static buffer that gets overwritten
+			 * by time functions. */
+			struct tm time = *time_ptr;
+
+			sprintf(state->str_fineta, "%.16s ", _("ETA"));
+			strftime(state->str_fineta +
+				 strlen(state->str_fineta),
+				 sizeof(state->str_fineta) - 1 -
+				 strlen(state->str_fineta),
+				 time_format, &time);
+		}
+
+		if (!show_eta) {
+			int i;
+			for (i = 0; i < sizeof(state->str_fineta)
+			     && state->str_fineta[i] != 0; i++) {
+				state->str_fineta[i] = ' ';
+			}
+		}
+	}
+
 	/*
 	 * Now go through all the static portions of the format to work
 	 * out how much space will be left for any dynamic portions
@@ -672,7 +775,7 @@ static char *pv__format(pvstate_t state,
 	 * Assemble the progress bar now we know how big it should be.
 	 */
 	if ((state->components_used & PV_DISPLAY_PROGRESS) != 0) {
-		char pct[16];		 /* RATS: ignore (big enough) */
+		char pct[16];
 		int available_width, i;
 
 		strcpy(state->str_progress, "[");
@@ -711,7 +814,7 @@ static char *pv__format(pvstate_t state,
 				strcat(state->str_progress, " ");
 			}
 			strcat(state->str_progress, "] ");
-			strcat(state->str_progress, pct);	/* RATS: ignore (OK) */
+			strcat(state->str_progress, pct);
 		} else {
 			int p = state->percentage;
 
@@ -780,7 +883,7 @@ static char *pv__format(pvstate_t state,
 		/* Skip segment if it would make the display too wide */
 		if (segment_length + display_string_length > state->width)
 			break;
-		strncat(state->display_buffer,	/* RATS: ignore */
+		strncat(state->display_buffer,
 			state->format[segment].string, segment_length);
 		display_string_length += segment_length;
 	}
@@ -792,7 +895,7 @@ static char *pv__format(pvstate_t state,
 	output_length = strlen(state->display_buffer);
 	if ((output_length < state->prev_length)
 	    && (state->width >= state->prev_width)) {
-		char spaces[32];	 /* RATS: ignore (bounded below) */
+		char spaces[32];
 		int spaces_to_add;
 		spaces_to_add = state->prev_length - output_length;
 		/* Upper boundary on number of spaces */
@@ -804,8 +907,7 @@ static char *pv__format(pvstate_t state,
 		while (--spaces_to_add >= 0) {
 			spaces[spaces_to_add] = ' ';
 		}
-		strcat(state->display_buffer,	/* RATS: ignore (OK) */
-		       spaces);
+		strcat(state->display_buffer, spaces);
 	}
 	state->prev_width = state->width;
 	state->prev_length = output_length;
@@ -849,12 +951,14 @@ void pv_display(pvstate_t state, long double esec, long long sl,
 		return;
 
 	if (state->numeric) {
-		write(STDERR_FILENO, display, strlen(display));	/* RATS: ignore */
+		write(STDERR_FILENO, display, strlen(display));
 	} else if (state->cursor) {
 		pv_crs_update(state, display);
+		state->display_visible = 1;
 	} else {
-		write(STDERR_FILENO, display, strlen(display));	/* RATS: ignore */
+		write(STDERR_FILENO, display, strlen(display));
 		write(STDERR_FILENO, "\r", 1);
+		state->display_visible = 1;
 	}
 
 	debug("%s: [%s]", "display", display);
