@@ -1,8 +1,6 @@
 /*
  * Main program entry point - read the command line options, then perform
  * the appropriate actions.
- *
- * Copyright 2014 Andrew Wood, distributed under the Artistic License 2.0.
  */
 
 #include "pv-internal.h"
@@ -88,8 +86,15 @@ int pv_main_loop(pvstate_t state)
 
 	next_update.tv_sec = start_time.tv_sec;
 	next_update.tv_usec = start_time.tv_usec;
-	pv_timeval_add_usec(&next_update,
-			    (long) (1000000.0 * state->interval));
+	if ((state->delay_start > 0)
+	    && (state->delay_start > state->interval)) {
+		pv_timeval_add_usec(&next_update,
+				    (long) (1000000.0 *
+					    state->delay_start));
+	} else {
+		pv_timeval_add_usec(&next_update,
+				    (long) (1000000.0 * state->interval));
+	}
 
 	next_ratecheck.tv_sec = start_time.tv_sec;
 	next_ratecheck.tv_usec = start_time.tv_usec;
@@ -215,7 +220,9 @@ int pv_main_loop(pvstate_t state)
 
 		if (eof_in && eof_out) {
 			final_update = 1;
-			next_update.tv_sec = cur_time.tv_sec - 1;
+			if ((state->display_visible)
+			    || (0 == state->delay_start))
+				next_update.tv_sec = cur_time.tv_sec - 1;
 		}
 
 		if (state->no_op)
@@ -311,7 +318,8 @@ int pv_main_loop(pvstate_t state)
 	if (state->cursor) {
 		pv_crs_fini(state);
 	} else {
-		if ((!state->numeric) && (!state->no_op))
+		if ((!state->numeric) && (!state->no_op)
+		    && (state->display_visible))
 			write(STDERR_FILENO, "\n", 1);
 	}
 
@@ -499,16 +507,15 @@ int pv_watchpid_loop(pvstate_t state)
 	int fd_to_idx[FD_SETSIZE] = { 0, };
 	struct timeval next_update, cur_time, next_remotecheck;
 	int idx;
-	int prev_displayed_lines;
+	int prev_displayed_lines, blank_lines;
 
 	/*
 	 * Make sure the process exists first, so we can give an error if
 	 * it's not there at the start.
 	 */
 	if (kill(state->watch_pid, 0) != 0) {
-		fprintf(stderr, "%s: %s %u: %s\n",
-			state->program_name,
-			_("pid"), state->watch_pid, strerror(errno));
+		pv_error(state, "%s %u: %s",
+			 _("pid"), state->watch_pid, strerror(errno));
 		state->exit_status |= 2;
 		return 2;
 	}
@@ -559,7 +566,7 @@ int pv_watchpid_loop(pvstate_t state)
 	prev_displayed_lines = 0;
 
 	while (1) {
-		int rc, fd, displayed_lines, blank_lines;
+		int rc, fd, displayed_lines;
 
 		if (state->pv_sig_abort)
 			break;
@@ -718,6 +725,24 @@ int pv_watchpid_loop(pvstate_t state)
 		}
 	}
 
+	/*
+	 * Clean up our displayed lines on exit.
+	 */
+	blank_lines = prev_displayed_lines;
+	while (blank_lines > 0) {
+		int x;
+		for (x = 0; x < state->width; x++)
+			write(STDERR_FILENO, " ", 1);
+		write(STDERR_FILENO, "\r", 1);
+		blank_lines--;
+		if (blank_lines > 0)
+			write(STDERR_FILENO, "\n", 1);
+	}
+	while (prev_displayed_lines > 1) {
+		write(STDERR_FILENO, "\033[A", 3);
+		prev_displayed_lines--;
+	}
+
 	if (NULL != info_array)
 		free(info_array);
 	if (NULL != state_array)
