@@ -1,7 +1,5 @@
 /*
  * Functions for transferring between file descriptors.
- *
- * Copyright 2014 Andrew Wood, distributed under the Artistic License 2.0.
  */
 
 #include "pv-internal.h"
@@ -18,11 +16,116 @@
 #include <signal.h>
 
 
+/*
+ * Read up to "count" bytes from file descriptor "fd" into the buffer "buf",
+ * and return the number of bytes read, like read().
+ *
+ * Unlike read(), if we have read less than "count" bytes, we check to see
+ * if there's any more to read, and keep trying, to make sure we fill the
+ * buffer as full as we can.
+ */
+static ssize_t pv__transfer_read_repeated(int fd, void *buf, size_t count)
+{
+	ssize_t total_read;
+
+	total_read = 0;
+
+	while (count > 0) {
+		ssize_t nread;
+
+		nread = read(fd, buf, count);
+		if (nread < 0)
+			return nread;
+
+		total_read += nread;
+		buf += nread;
+		count -= nread;
+
+		if (0 == nread)
+			return total_read;
+
+		if (count > 0) {
+			fd_set readfds;
+			struct timeval tv;
+
+			tv.tv_sec = 0;
+			tv.tv_usec = 0;
+			FD_ZERO(&readfds);
+			FD_SET(fd, &readfds);
+
+			debug("%s %d: %s (%ld %s, %ld %s)", "fd", fd,
+			      "trying another read after partial buffer fill",
+			      nread, "read", count, "remaining");
+
+			if (select(fd + 1, &readfds, NULL, NULL, &tv) < 1)
+				break;
+		}
+	}
+
+	return total_read;
+}
+
+
+/*
+ * Write up to "count" bytes to file descriptor "fd" from the buffer "buf",
+ * and return the number of bytes written, like write().
+ *
+ * Unlike write(), if we have written less than "count" bytes, we check to
+ * see if we can write any more, and keep trying, to make sure we empty the
+ * buffer as much as we can.
+ */
+static ssize_t pv__transfer_write_repeated(int fd, void *buf, size_t count)
+{
+	ssize_t total_written;
+
+	total_written = 0;
+
+	while (count > 0) {
+		ssize_t nwritten;
+
+		nwritten = write(fd, buf, count);
+		if (nwritten < 0)
+			return nwritten;
+
+		total_written += nwritten;
+		buf += nwritten;
+		count -= nwritten;
+
+		if (0 == nwritten)
+			return total_written;
+
+		if (count > 0) {
+			fd_set writefds;
+			struct timeval tv;
+
+			tv.tv_sec = 0;
+			tv.tv_usec = 0;
+			FD_ZERO(&writefds);
+			FD_SET(fd, &writefds);
+
+			debug("%s %d: %s (%ld %s, %ld %s)", "fd", fd,
+			      "trying another write after partial buffer flush",
+			      nwritten, "written", count, "remaining");
+
+			if (select(fd + 1, NULL, &writefds, NULL, &tv) < 1)
+				break;
+		}
+	}
+
+	return total_written;
+}
+
+
 /*
  * Read some data from the given file descriptor. Returns zero if there was
  * a transient error and we need to return 0 from pv_transfer, otherwise
  * returns 1.
  *
+ * At most, the number of bytes read will be the number of bytes remaining
+ * in the input buffer.  If state->rate_limit is >0, and/or "allowed" is >0,
+ * then the maximum number of bytes read will be the number remaining unused
+ * in the input buffer or the value of "allowed", whichever is smaller.
+ *
  * If splice() was successfully used, sets state->splice_used to 1; if it
  * failed, then state->splice_failed_fd is updated to the current fd so
  * splice() won't be tried again until the next input file.
@@ -71,6 +174,8 @@ static int pv__transfer_read(pvstate_t state, int fd,
 
 		state->splice_used = 1;
 		if ((nread < 0) && (EINVAL == errno)) {
+			debug("%s %d: %s", "fd", fd,
+			      "splice failed with EINVAL - disabling");
 			state->splice_failed_fd = fd;
 			state->splice_used = 0;
 			/*
@@ -86,14 +191,18 @@ static int pv__transfer_read(pvstate_t state, int fd,
 		}
 	}
 	if (0 == state->splice_used) {
-		nread = read( /* RATS: ignore (checked OK) */ fd,
-			     state->transfer_buffer + state->read_position,
-			     bytes_can_read);
+		nread =
+		    pv__transfer_read_repeated(fd,
+					       state->transfer_buffer +
+					       state->read_position,
+					       bytes_can_read);
 	}
 #else
-	nread = read( /* RATS: ignore (checked OK) */ fd,
-		     state->transfer_buffer + state->read_position,
-		     bytes_can_read);
+	nread =
+	    pv__transfer_read_repeated(fd,
+				       state->transfer_buffer +
+				       state->read_position,
+				       bytes_can_read);
 #endif				/* HAVE_SPLICE */
 
 
@@ -138,6 +247,9 @@ static int pv__transfer_read(pvstate_t state, int fd,
 	 */
 	if ((EINTR == errno) || (EAGAIN == errno)) {
 		struct timeval tv;
+		debug("%s %d: %s: %s", "fd", fd,
+		      "transient error - waiting briefly",
+		      strerror(errno));
 		tv.tv_sec = 0;
 		tv.tv_usec = 10000;
 		select(0, NULL, NULL, NULL, &tv);
@@ -157,10 +269,9 @@ static int pv__transfer_read(pvstate_t state, int fd,
 	 * reached the end of this file.
 	 */
 	if (0 == state->skip_errors) {
-		fprintf(stderr, "%s: %s: %s: %s\n",
-			state->program_name,
-			state->current_file,
-			_("read failed"), strerror(errno));
+		pv_error(state, "%s: %s: %s",
+			 state->current_file,
+			 _("read failed"), strerror(errno));
 		*eof_in = 1;
 		if (state->write_position >= state->read_position) {
 			*eof_out = 1;
@@ -175,12 +286,11 @@ static int pv__transfer_read(pvstate_t state, int fd,
 	amount_skipped = -1;
 
 	if (!state->read_error_warning_shown) {
-		fprintf(stderr, "%s: %s: %s: %s\n",
-			state->program_name,
-			state->current_file,
-			_
-			("warning: read errors detected"),
-			strerror(errno));
+		pv_error(state, "%s: %s: %s",
+			 state->current_file,
+			 _
+			 ("warning: read errors detected"),
+			 strerror(errno));
 		state->read_error_warning_shown = 1;
 	}
 
@@ -192,10 +302,9 @@ static int pv__transfer_read(pvstate_t state, int fd,
 	 * of the file.
 	 */
 	if (0 > orig_offset) {
-		fprintf(stderr, "%s: %s: %s: %s\n",
-			state->program_name,
-			state->current_file,
-			_("file is not seekable"), strerror(errno));
+		pv_error(state, "%s: %s: %s",
+			 state->current_file,
+			 _("file is not seekable"), strerror(errno));
 		*eof_in = 1;
 		if (state->write_position >= state->read_position) {
 			*eof_out = 1;
@@ -254,13 +363,12 @@ static int pv__transfer_read(pvstate_t state, int fd,
 		 * since it just means we've reached the end anyway.
 		 */
 		if (EINVAL != errno) {
-			fprintf(stderr,
-				"%s: %s: %s: %s\n",
-				state->program_name,
-				state->current_file,
-				_
-				("failed to seek past error"),
-				strerror(errno));
+			pv_error(state,
+				 "%s: %s: %s",
+				 state->current_file,
+				 _
+				 ("failed to seek past error"),
+				 strerror(errno));
 		}
 	} else {
 		amount_skipped = skip_offset - orig_offset;
@@ -275,14 +383,12 @@ static int pv__transfer_read(pvstate_t state, int fd,
 		       state->read_position, 0, amount_skipped);
 		state->read_position += amount_skipped;
 		if (state->skip_errors < 2) {
-			fprintf(stderr,
-				"%s: %s: %s: %ld - %ld (%ld %s)\n",
-				state->program_name,
-				state->current_file,
-				_
-				("skipped past read error"),
-				orig_offset,
-				skip_offset, amount_skipped, _("B"));
+			pv_error(state, "%s: %s: %ld - %ld (%ld %s)",
+				 state->current_file,
+				 _
+				 ("skipped past read error"),
+				 orig_offset,
+				 skip_offset, amount_skipped, _("B"));
 		}
 	} else {
 		/*
@@ -315,24 +421,25 @@ static int pv__transfer_write(pvstate_t state, int fd,
 			      int *eof_in, int *eof_out,
 			      long *lineswritten)
 {
-	ssize_t nwrote;
+	ssize_t nwritten;
 
-	signal(SIGALRM, SIG_IGN);	    /* RATS: ignore */
+	signal(SIGALRM, SIG_IGN);
 	alarm(1);
 
-	nwrote = write(STDOUT_FILENO,
-		       state->transfer_buffer + state->write_position,
-		       state->to_write);
+	nwritten = pv__transfer_write_repeated(STDOUT_FILENO,
+					       state->transfer_buffer +
+					       state->write_position,
+					       state->to_write);
 
 	alarm(0);
 
-	if (0 == nwrote) {
+	if (0 == nwritten) {
 		/*
 		 * Write returned 0 - EOF on stdout.
 		 */
 		*eof_out = 1;
 		return 1;
-	} else if (nwrote > 0) {
+	} else if (nwritten > 0) {
 		/*
 		 * Write returned >0 - data successfully written.
 		 */
@@ -346,9 +453,9 @@ static int pv__transfer_write(pvstate_t state, int fd,
 
 			save =
 			    state->transfer_buffer[state->write_position +
-						   nwrote];
+						   nwritten];
 			state->transfer_buffer[state->write_position +
-					       nwrote] = 0;
+					       nwritten] = 0;
 			ptr =
 			    (char *) (state->transfer_buffer +
 				      state->write_position - 1);
@@ -357,7 +464,7 @@ static int pv__transfer_write(pvstate_t state, int fd,
 				for (ptr++;
 				     ptr -
 				     (char *) state->transfer_buffer -
-				     state->write_position < nwrote;
+				     state->write_position < nwritten;
 				     ptr++) {
 					if (*ptr == '\0')
 						++lines;
@@ -370,21 +477,21 @@ static int pv__transfer_write(pvstate_t state, int fd,
 
 			*lineswritten += lines;
 			state->transfer_buffer[state->write_position +
-					       nwrote] = save;
+					       nwritten] = save;
 		}
 
-		state->write_position += nwrote;
-		state->written += nwrote;
+		state->write_position += nwritten;
+		state->written += nwritten;
 
 		/*
 		 * If we're monitoring the output, update our copy of the
 		 * last few bytes we've written.
 		 */
 		if (((state->components_used & PV_DISPLAY_OUTPUTBUF) != 0)
-		    && (nwrote > 0)) {
+		    && (nwritten > 0)) {
 			long new_portion_length, old_portion_length;
 
-			new_portion_length = nwrote;
+			new_portion_length = nwritten;
 			if (new_portion_length > state->lastoutput_length)
 				new_portion_length =
 				    state->lastoutput_length;
@@ -429,7 +536,7 @@ static int pv__transfer_write(pvstate_t state, int fd,
 	}
 
 	/*
-	 * If we reach this point, nwrote<0, so there was an error.
+	 * If we reach this point, nwritten<0, so there was an error.
 	 */
 
 	/*
@@ -454,8 +561,7 @@ static int pv__transfer_write(pvstate_t state, int fd,
 		return 0;
 	}
 
-	fprintf(stderr, "%s: %s: %s\n",
-		state->program_name, _("write failed"), strerror(errno));
+	pv_error(state, "%s: %s", _("write failed"), strerror(errno));
 	state->exit_status |= 16;
 	*eof_out = 1;
 	state->written = -1;
@@ -502,10 +608,9 @@ long pv_transfer(pvstate_t state, int fd, int *eof_in, int *eof_out,
 		state->transfer_buffer =
 		    (unsigned char *) malloc(state->buffer_size + 32);
 		if (NULL == state->transfer_buffer) {
-			fprintf(stderr, "%s: %s: %s\n",
-				state->program_name,
-				_("buffer allocation failed"),
-				strerror(errno));
+			pv_error(state, "%s: %s",
+				 _("buffer allocation failed"),
+				 strerror(errno));
 			state->exit_status |= 64;
 			return -1;
 		}
@@ -517,15 +622,18 @@ long pv_transfer(pvstate_t state, int fd, int *eof_in, int *eof_out,
 	if (state->buffer_size < state->target_buffer_size) {
 		unsigned char *newptr;
 		newptr =
-		    realloc( /* RATS: ignore */ state->transfer_buffer,
+		    realloc(state->transfer_buffer,
 			    state->target_buffer_size + 32);
 		if (NULL == newptr) {
 			/*
 			 * Reset target if realloc failed so we don't keep
 			 * trying to realloc over and over.
 			 */
+			debug("realloc: %s", strerror(errno));
 			state->target_buffer_size = state->buffer_size;
 		} else {
+			debug("%s: %ld", "buffer resized",
+			      state->buffer_size);
 			state->transfer_buffer = newptr;
 			state->buffer_size = state->target_buffer_size;
 		}
@@ -590,9 +698,9 @@ long pv_transfer(pvstate_t state, int fd, int *eof_in, int *eof_out,
 		/*
 		 * Any other error is a problem and we must report back.
 		 */
-		fprintf(stderr, "%s: %s: %s: %d: %s\n",
-			state->program_name, state->current_file,
-			_("select call failed"), n, strerror(errno));
+		pv_error(state, "%s: %s: %d: %s",
+			 state->current_file,
+			 _("select call failed"), n, strerror(errno));
 
 		state->exit_status |= 16;
 
